/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Michael de Hart <mdhart@intermesh.nl>
 */
import { Component, ComponentEventMap } from "../Component.js";
import { DateTime } from "../../util/DateTime.js";
import { Config, ObservableListenerOpts } from "../Observable.js";
export interface DatePickerEventMap<Type> extends ComponentEventMap<Type> {
    'select': (datepicker: Type, date: DateTime) => false | void;
    'select-range': (datepicker: Type, start: DateTime, end: DateTime) => false | void;
}
export interface DatePicker {
    on<K extends keyof DatePickerEventMap<this>>(eventName: K, listener: Partial<DatePickerEventMap<this>>[K], options?: ObservableListenerOpts): void;
    fire<K extends keyof DatePickerEventMap<this>>(eventName: K, ...args: Parameters<DatePickerEventMap<Component>[K]>): boolean;
}
export declare class DatePicker extends Component {
    showWeekNbs: boolean;
    value: DateTime;
    private now;
    enableRangeSelect: boolean;
    protected baseCls: string;
    protected monthEl: HTMLSpanElement;
    protected yearEl: HTMLSpanElement;
    protected years: HTMLElement;
    protected months: HTMLElement;
    protected grid: HTMLElement;
    protected menu: HTMLElement;
    minDate?: DateTime;
    maxDate?: DateTime;
    constructor();
    moveMonth(amount: number): void;
    protected internalRender(): HTMLElement;
    setValue(start: DateTime, end?: DateTime): void;
    private markSelected;
    private setupDraggability;
}
export declare const datepicker: (config?: Config<DatePicker, DatePickerEventMap<DatePicker>>) => DatePicker;
//# sourceMappingURL=DatePicker.d.ts.map