/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Michael de Hart <mdhart@intermesh.nl>
 */
import { Component } from "../Component.js";
import { DateTime } from "../../util/DateTime.js";
import { Form } from "../form/Form.js";
import { Frequency, RecurrenceRule } from "../../util/Recurrence.js";
import { ObservableListenerOpts } from "../Observable.js";
import { CardContainer, CardContainerEventMap } from "../CardContainer.js";
export interface RecurrencePickerEventMap<Type> extends CardContainerEventMap<Type> {
    select: (picker: Type, rule: RecurrenceRule | null) => false | void;
}
export interface RecurrencePicker {
    on<K extends keyof RecurrencePickerEventMap<this>>(eventName: K, listener: Partial<RecurrencePickerEventMap<this>>[K], options?: ObservableListenerOpts): void;
    fire<K extends keyof RecurrencePickerEventMap<this>>(eventName: K, ...args: Parameters<RecurrencePickerEventMap<Component>[K]>): boolean;
}
type FrequencyDefaults = [text: string, plural: string, repeatDefault: number, untilDefault: string, frequencyText: string];
export declare class RecurrencePicker extends CardContainer {
    protected baseCls: string;
    form: Form;
    menu: Component;
    endAtCtr: Component;
    weekOfMonth: number;
    startDate: DateTime;
    rule: RecurrenceRule | null;
    static frequencies: {
        [freq: string]: FrequencyDefaults;
    };
    constructor(startDate: DateTime);
    quickMenuItems(): (Component | import("../Button.js").Button)[];
    createCustomRule(values: any): RecurrenceRule;
    setStartDate(date: DateTime): void;
    changeFrequency(f: Frequency): void;
    private getSuffix;
    setValue(rrule: RecurrenceRule | null): void;
}
export {};
//# sourceMappingURL=RecurrencePicker.d.ts.map