import { BaseEntity, Component, ComponentEventMap, DefaultEntity, ObservableListenerOpts, Toolbar } from "@intermesh/goui";
export interface DetailPanelEventMap<Type, EntityType extends BaseEntity = DefaultEntity> extends ComponentEventMap<Type> {
    /**
     * Fires when entity is loaded
     */
    load: (detailPanel: Type, entity: EntityType) => false | void;
}
export interface DetailPanel<EntityType extends BaseEntity = DefaultEntity> extends Component {
    on<K extends keyof DetailPanelEventMap<this, EntityType>>(eventName: K, listener: Partial<DetailPanelEventMap<this, EntityType>>[K], options?: ObservableListenerOpts): void;
    fire<K extends keyof DetailPanelEventMap<this, EntityType>>(eventName: K, ...args: Parameters<DetailPanelEventMap<any, EntityType>[K]>): boolean;
}
/**
 * Detail panel
 *
 * Used to show an entity when selected in the grid.
 */
export declare abstract class DetailPanel<EntityType extends BaseEntity = DefaultEntity> extends Component {
    entityName: string;
    private titleCmp;
    protected entity?: EntityType;
    protected readonly scroller: Component;
    private detailView;
    protected toolbar: Toolbar;
    protected constructor(entityName: string);
    private get legacyDetailView();
    protected addCustomFields(): void;
    protected addLinks(): void;
    protected addComments(): void;
    protected addFiles(): void;
    protected addHistory(): void;
    private createToolbar;
    set title(title: string);
    load(id: string): Promise<this>;
    private legacyOnLoad;
}
//# sourceMappingURL=DetailPanel.d.ts.map