import { CardContainer, DataSourceForm, EntityID, Fieldset, ObservableListenerOpts, Window, WindowEventMap } from "@intermesh/goui";
import { SharePanel } from "../permissions";
/**
 * @inheritDoc
 */
export interface FormWindowEventMap<Type> extends WindowEventMap<Type> {
    /**
     * Fires when the window is shown and loaded with data. Also fires if the dialog is for creating new entities and
     * not loaded.
     *
     * @param window
     */
    ready: (window: Type, currentId: string | undefined) => void;
}
export interface FormWindow {
    on<K extends keyof FormWindowEventMap<this>>(eventName: K, listener: Partial<FormWindowEventMap<this>>[K], options?: ObservableListenerOpts): void;
    fire<K extends keyof FormWindowEventMap<this>>(eventName: K, ...args: Parameters<FormWindowEventMap<any>[K]>): boolean;
}
export declare abstract class FormWindow extends Window {
    entityName: string;
    readonly form: DataSourceForm;
    protected currentId?: EntityID;
    protected readonly cards: CardContainer;
    protected sharePanel?: SharePanel;
    /**
     * The first tab
     *
     * @protected
     */
    protected readonly generalTab: Fieldset;
    /**
     * Constructor
     *
     * @param entityName Name of the entity for the datasource form
     * @protected
     */
    protected constructor(entityName: string);
    /**
     * Add a share panel to set permissions
     *
     * @protected
     */
    protected addSharePanel(): void;
    load(id: EntityID): Promise<this>;
    protected addCustomFields(): void;
    private renderCustomFields;
}
//# sourceMappingURL=FormWindow.d.ts.map