import { AutocompleteField } from "./AutocompleteField";
import { AbstractDataSource, DataSourceStore, DataSourceStoreConfig, StoreRecord } from "../../data";
import { Table } from "../table";
import { Config } from "../Observable";
import { FieldEventMap } from "./Field";
/**
 * Combo box
 *
 * An extension of the Autocomplete field that simplifies the creation of a simple combobox with a
 * valueProperty and displayProperty.
 */
export declare class ComboBox<DS extends AbstractDataSource = AbstractDataSource> extends AutocompleteField<Table<DataSourceStore<DS>>> {
    readonly dataSource: DS;
    readonly displayProperty: string;
    readonly valueProperty: string;
    filterName?: string;
    constructor(dataSource: DS, displayProperty?: string, valueProperty?: string);
    /**
     * Pass config options for the underlying dropdown table store
     * @param storeConfig
     */
    set storeConfig(storeConfig: Omit<DataSourceStoreConfig<DS, StoreRecord>, "dataSource">);
    pickerRecordToValue(_field: this, record: any): string;
    valueToTextField(_field: this, value: string): Promise<string>;
}
type ComboBoxConfig = Config<ComboBox, FieldEventMap<ComboBox>, "dataSource">;
/**
 * Shorthand function to create {@see DateField}
 *
 * @param config
 */
export declare const combobox: (config: ComboBoxConfig) => ComboBox<AbstractDataSource<import("../../data").DefaultEntity>>;
export {};
//# sourceMappingURL=ComboBox.d.ts.map