import { AutocompleteField } from "./AutocompleteField";
import { datasourcestore } from "../../data";
import { column, table } from "../table";
import { createComponent } from "../Component";
import { ObjectUtil } from "../../util";
/**
 * Combo box
 *
 * An extension of the Autocomplete field that simplifies the creation of a simple combobox with a
 * valueProperty and displayProperty.
 */
export class ComboBox extends AutocompleteField {
    constructor(dataSource, displayProperty = "name", valueProperty = "id") {
        const dropDownTable = table({
            headers: false,
            fitParent: true,
            store: datasourcestore({
                dataSource: dataSource,
                queryParams: {
                    limit: 50
                }
            }),
            columns: [
                column({
                    id: displayProperty,
                    resizable: true,
                    width: 312,
                    sortable: true
                })
            ]
        });
        super(dropDownTable);
        this.dataSource = dataSource;
        this.displayProperty = displayProperty;
        this.valueProperty = valueProperty;
        // load next data on scroll
        this.picker.list.store.addScrollLoader(this.menu.el);
        this.on("autocomplete", async (_field, input) => {
            this.list.store.queryParams.position = 0;
            if (!this.list.store.queryParams.filter) {
                this.list.store.queryParams.filter = {};
            }
            this.list.store.queryParams.filter[this.filterName ?? this.displayProperty] = input;
            await this.list.store.load();
        });
    }
    /**
     * Pass config options for the underlying dropdown table store
     * @param storeConfig
     */
    set storeConfig(storeConfig) {
        ObjectUtil.merge(this.picker.list.store, storeConfig);
    }
    pickerRecordToValue(_field, record) {
        return record[this.valueProperty];
    }
    async valueToTextField(_field, value) {
        if (value === "") {
            return "";
        }
        const nb = await this.dataSource.single(value);
        return nb ? nb[this.displayProperty] : "?";
    }
}
/**
 * Shorthand function to create {@see DateField}
 *
 * @param config
 */
export const combobox = (config) => createComponent(new ComboBox(config.dataSource, config.displayProperty ?? "name", config.valueProperty ?? "id"), config);
//# sourceMappingURL=ComboBox.js.map