/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Michael de Hart <mdhart@intermesh.nl>
 */
import { Field } from "./Field.js";
import { createComponent } from "../Component.js";
import { btn } from "../Button.js";
export class MapField extends Field {
    constructor(buildField) {
        super('div');
        this.buildField = buildField;
        this.rows = [];
        this.baseCls = "";
        this._nextKey = 1;
    }
    renderControl() {
        // empty
    }
    set value(v) {
        super.value = v;
        this.items.clear();
        if (v) {
            for (const key in v) {
                this.internalAdd(v[key], key);
            }
        }
    }
    get value() {
        const v = {};
        this.rows.forEach(row => {
            v[row.key] = row.field.value;
        });
        return v;
    }
    /**
     * Add value to the map.
     *
     * Also fires change event
     *
     * @param data
     * @param key
     */
    add(data, key) {
        if (!this.oldValue) {
            this.captureValueForChange();
        }
        this.internalAdd(data, key);
        this.fireChange();
    }
    internalAdd(data, key) {
        const row = {
            key: key === undefined ? this.nextKey() : key,
            field: this.buildField(data),
            isNew: true
        };
        row.field.items.add(btn({
            icon: "delete",
            handler: button => {
                this.rows.splice(this.rows.indexOf(row), 1);
                row.field.remove();
            }
        }));
        row.field.value = data;
        this.rows.push(row);
        this.items.add(row.field);
    }
    nextKey() {
        // only works if sorted by key
        // todo: maybe use GUID for item with sortOrder?
        return "_new_" + this._nextKey++;
    }
    reset() {
        super.reset();
        this.rows = [];
        this.items.clear();
    }
}
export const mapfield = (config, ...items) => createComponent(new MapField(config.buildField), config, items);
//# sourceMappingURL=MapField.js.map