/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Merijn Schering <mschering@intermesh.nl>
 */
import { TextField } from "./TextField.js";
import { createComponent } from "../Component.js";
export class NumberField extends TextField {
    constructor() {
        super(...arguments);
        this.type = "number";
        this.decimals = 2;
    }
    validate() {
        super.validate();
        const v = super.value;
        if (v && isNaN(v)) {
            this.setInvalid("Incorrect number format");
        }
        if (this.max !== undefined && v > this.max) {
            this.setInvalid("Number is too big");
        }
        if (this.min !== undefined && v < this.min) {
            this.setInvalid("Number is too small");
        }
    }
    set value(v) {
        if (isNaN(v)) {
            throw new Error("Invalid number");
        }
        else if (v) {
            super.value = +v.toFixed(this.decimals);
        }
    }
    get value() {
        const v = super.value;
        return (v === undefined || isNaN(v)) ? undefined : +(+v).toFixed(this.decimals);
    }
    createControl() {
        super.createControl();
        this._input.cls('+number');
        // this._input!.attr('type','number');
        if (this.min !== undefined) {
            this._input.attr('min', this.min);
        }
        if (this.max !== undefined) {
            this._input.attr('max', this.max);
        }
        if (this.decimals) {
            this._input.attr('step', '0.'.padEnd(this.decimals + 1, "0") + "1");
        }
        return this._input;
    }
}
export const numberfield = (config) => createComponent(new NumberField(), config);
//# sourceMappingURL=NumberField.js.map