/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Merijn Schering <mschering@intermesh.nl>
 */
import { Field } from "./Field.js";
import { createComponent } from "../Component.js";
/**
 * Select field
 *
 * @see Form
 */
export class SelectField extends Field {
    constructor() {
        super(...arguments);
        this.baseCls = "goui-form-field select";
        this.options = [];
        this.valueField = 'value';
        this.textRenderer = (record) => record.name;
    }
    createControl() {
        //grab value before creating this.input otherwise it will return the input value
        const v = this.value;
        this.input = document.createElement("select");
        this.input.name = this.name;
        if (this.required) {
            this.input.setAttribute("required", "");
        }
        this.drawOptions();
        this.el.appendChild(this.input);
        return this.input;
    }
    getInput() {
        return this.input;
    }
    drawOptions() {
        if (!this.input)
            return;
        this.input.innerHTML = ''; // redraw
        (this.store ? this.store.items : this.options).forEach((o) => {
            const opt = new Option();
            if (o[this.valueField]) {
                opt.value = o[this.valueField];
            }
            opt.innerHTML = this.textRenderer(o);
            this.input?.appendChild(opt);
        });
        if (this._value) {
            // for updating this.input.selectIndex
            this.value = this._value;
        }
    }
    set value(v) {
        if (this.input) {
            this.input.value = v;
        }
        super.value = v;
    }
    get value() {
        if (!this.input) {
            return super.value;
        }
        else {
            return this.input.value;
        }
    }
    set name(name) {
        super.name = (name);
        if (this.input) {
            this.input.name = this.name;
        }
    }
    get name() {
        return super.name;
    }
    validate() {
        super.validate();
        //this implements the native browser validation
        if (this.input)
            this.setValidityState(this.input);
    }
}
/**
 * Shorthand function to create {@see SelectField}
 *
 * @param config
 */
export const select = (config) => createComponent(new SelectField(), config);
//# sourceMappingURL=SelectField.js.map