/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Merijn Schering <mschering@intermesh.nl>
 */
import { Field, FieldEventMap } from "./Field.js";
import { Config } from "../Observable";
export type TextFieldType = ("text" | "password" | "email" | "url" | "tel" | "search" | "time" | "date" | "datetime-local");
/**
 * TextField component
 *
 * @see Form
 */
export declare class TextField extends Field {
    protected _input: HTMLInputElement | HTMLTextAreaElement | undefined;
    /**
     * input type. text, password, email etc.
     */
    type: TextFieldType;
    /**
     * When the field is empty this will be displayed inside the field
     */
    placeholder: string;
    /**
     * Autocomplete value
     */
    autocomplete: AutoFill | undefined;
    /**
     * Pattern regex for validation
     */
    pattern: HTMLInputElement["pattern"] | undefined;
    get input(): HTMLInputElement | HTMLTextAreaElement | undefined;
    set title(title: string);
    focus(o?: FocusOptions): void;
    protected createControl(): undefined | HTMLElement;
    setInvalid(msg: string): void;
    clearInvalid(): void;
    protected setInputValue(v: string): void;
    set value(v: any);
    get value(): any;
    set name(name: string);
    get name(): string;
    set readOnly(readOnly: boolean);
    get readOnly(): boolean;
    protected validate(): void;
}
/**
 * Shorthand function to create {@see TextField}
 *
 * @param config
 */
export declare const textfield: (config?: Config<TextField, FieldEventMap<TextField>>) => TextField;
//# sourceMappingURL=TextField.d.ts.map