/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Merijn Schering <mschering@intermesh.nl>
 */
import { Field } from "./Field.js";
import { createComponent } from "../Component.js";
/**
 * TextField component
 *
 * @see Form
 */
export class TextField extends Field {
    constructor() {
        super(...arguments);
        /**
         * input type. text, password, email etc.
         */
        this.type = "text";
        /**
         * When the field is empty this will be displayed inside the field
         */
        this.placeholder = " ";
    }
    get input() {
        return this._input;
    }
    set title(title) {
        super.title = title;
        if (this._input) {
            this._input.title = this.title;
        }
    }
    focus(o) {
        if (!this._input) {
            super.focus(o);
        }
        this._input?.focus(o);
    }
    createControl() {
        //grab value before creating this.input otherwise it will return the input value
        const v = this.value, name = this.name;
        this._input = document.createElement("input");
        this._input.classList.add("text");
        this._input.type = this.type;
        if (this.pattern) {
            this._input.pattern = this.pattern;
        }
        if (this.autocomplete) {
            this._input.autocomplete = this.autocomplete;
        }
        if (this.placeholder) {
            this._input.placeholder = this.placeholder;
        }
        this._input.required = this.required;
        if (name) {
            this._input.name = name;
        }
        this._input.readOnly = this.readOnly;
        if (v !== undefined) {
            this._input.value = v;
        }
        if (this.title) {
            this._input.title = this.title;
        }
        if (this.invalidMsg) {
            this.applyInvalidMsg();
        }
        return this._input;
    }
    setInvalid(msg) {
        super.setInvalid(msg);
        if (this.rendered) {
            this.applyInvalidMsg();
        }
    }
    clearInvalid() {
        super.clearInvalid();
        this.applyInvalidMsg();
    }
    setInputValue(v) {
        if (this._input) {
            this._input.value = v + "";
        }
    }
    set value(v) {
        this.setInputValue(v || "");
        super.value = v;
    }
    get value() {
        if (!this._input) {
            return super.value;
        }
        else {
            return this._input.value;
        }
    }
    set name(name) {
        super.name = name;
        if (this._input) {
            this._input.name = this.name;
        }
    }
    get name() {
        return super.name;
    }
    set readOnly(readOnly) {
        super.readOnly = readOnly;
        if (this._input) {
            this._input.readOnly = this.readOnly;
        }
    }
    get readOnly() {
        return super.readOnly;
    }
    validate() {
        super.validate();
        //this implements the native browser validation
        if (this._input) {
            this.setValidityState(this._input);
        }
    }
}
/**
 * Shorthand function to create {@see TextField}
 *
 * @param config
 */
export const textfield = (config) => createComponent(new TextField(), config);
//# sourceMappingURL=TextField.js.map