import { comp, Component, FunctionUtil, t, tbar, Window } from "@intermesh/goui";
import { jmapds } from "../jmap";
/**
 * Detail panel
 *
 * Used to show an entity when selected in the grid.
 */
export class DetailPanel extends Component {
    constructor(entityName) {
        super();
        this.entityName = entityName;
        // reload on entity change
        jmapds(this.entityName).on("change", (ds, changes) => {
            if (this.entity && changes.updated && changes.updated.indexOf(this.entity.id + "") > -1) {
                this.load(this.entity.id);
            }
        });
        this.cls = "vbox";
        this.width = 400;
        this.style.position = "relative";
        this.items.add(this.createToolbar(), this.scroller = comp({ flex: 1, cls: "scroll vbox" }));
    }
    get legacyDetailView() {
        if (!this.detailView) {
            if ("ResizeObserver" in window) {
                const ro = new ResizeObserver(FunctionUtil.buffer(100, () => {
                    this.detailView.doLayout();
                }));
                ro.observe(this.el);
            }
            this.detailView = new go.detail.Panel({
                width: undefined,
                entityStore: go.Db.store(this.entityName),
                header: false
            });
            this.scroller.items.add(this.detailView);
        }
        return this.detailView;
    }
    addCustomFields() {
        this.legacyDetailView.addCustomFields();
    }
    addLinks() {
        this.legacyDetailView.addLinks();
    }
    addComments() {
        this.legacyDetailView.addComments();
    }
    addFiles() {
        this.legacyDetailView.addFiles();
    }
    addHistory() {
        this.legacyDetailView.addHistory();
    }
    createToolbar() {
        return this.toolbar = tbar({
            disabled: true,
            cls: "border-bottom"
        }, this.titleCmp = comp({ tagName: "h3" }), '->');
    }
    set title(title) {
        super.title = title;
        this.titleCmp.text = title;
    }
    async load(id) {
        this.mask();
        try {
            this.entity = await jmapds(this.entityName).single(id);
            if (!this.entity) {
                throw "notfound";
            }
            this.fire("load", this, this.entity);
            // this.title = this.entity.name;
            //
            // this.content.items.clear();
            // this.content.items.add(Image.replace(this.entity.content));
            this.legacyOnLoad();
            this.toolbar.disabled = false;
        }
        catch (e) {
            void Window.alert(t("Error"), e + "");
        }
        finally {
            this.unmask();
        }
        return this;
    }
    legacyOnLoad() {
        this.detailView.currentId = this.entity.id;
        this.detailView.internalLoad(this.entity);
    }
}
//# sourceMappingURL=DetailPanel.js.map