import { btn, cardmenu, cards, containerfield, datasourceform, fieldset, t, tbar, Window } from "@intermesh/goui";
import { sharepanel } from "../permissions";
import { jmapds } from "../jmap";
export class FormWindow extends Window {
    /**
     * Constructor
     *
     * @param entityName Name of the entity for the datasource form
     * @protected
     */
    constructor(entityName) {
        super();
        this.entityName = entityName;
        this.cls = "vbox";
        this.width = 600;
        this.height = 600;
        this.items.add(this.form = datasourceform({
            dataSource: jmapds(this.entityName),
            cls: "vbox",
            flex: 1,
            listeners: {
                save: () => {
                    this.close();
                },
                invalid: (form) => {
                    const invalid = form.findFirstInvalid();
                    if (invalid) {
                        const tab = invalid.findAncestor(cmp => {
                            return cmp.el.classList.contains('card-container-item');
                        });
                        if (tab) {
                            tab.show();
                        }
                        invalid.focus();
                    }
                }
            }
        }, cardmenu(), this.cards = cards({ flex: 1 }, this.generalTab = fieldset({
            cls: "scroll fit",
            title: t("General")
        })), tbar({ cls: "border-top" }, "->", btn({
            type: "submit",
            text: t("Save")
        }))));
        // fire the ready event if not loading the form with data. If it's loading then the ready event will fire
        // on the form load event.
        this.on("show", () => {
            setTimeout(() => {
                if (!this.currentId) {
                    // focus form for new entities and not for existing ones.
                    this.form.focus();
                    this.fire("ready", this, this.currentId);
                }
            });
        });
    }
    /**
     * Add a share panel to set permissions
     *
     * @protected
     */
    addSharePanel() {
        this.sharePanel = sharepanel({
            cls: "fit",
            listeners: {
                show: () => {
                    this.sharePanel.load();
                }
            }
        });
        this.cards.items.add(this.sharePanel);
        this.on("ready", () => {
            this.sharePanel.setEntity(this.entityName, this.currentId);
        });
    }
    async load(id) {
        this.mask();
        try {
            this.currentId = id;
            await this.form.load(id);
            this.fire("ready", this, this.currentId);
        }
        catch (e) {
            void Window.alert(t("Error"), e + "");
        }
        finally {
            this.unmask();
        }
        return this;
    }
    addCustomFields() {
        this.on("render", () => {
            if (this.hidden) {
                this.on("show", () => this.renderCustomFields());
            }
            else {
                this.renderCustomFields();
            }
        });
    }
    renderCustomFields() {
        if (go.Entities.get(this.entityName).customFields) {
            const fieldsets = go.customfields.CustomFields.getFormFieldSets(this.entityName);
            fieldsets.forEach((fs) => {
                //replace customFields. because we will use a containerfield here.
                fs.cascade((item) => {
                    if (item.getName) {
                        let fieldName = item.getName().replace('customFields.', '');
                        item.name = item.hiddenName = fieldName;
                    }
                });
                if (fs.fieldSet.isTab) {
                    fs.title = null;
                    fs.collapsible = false;
                    this.cards.items.add(containerfield({ name: "customFields", cls: "scroll", title: fs.fieldSet.name }, fs));
                }
                else {
                    //in case formPanelLayout is set to column
                    fs.columnWidth = 1;
                    this.generalTab.items.add(containerfield({ name: "customFields" }, fs));
                }
            }, this);
        }
    }
}
//# sourceMappingURL=FormWindow.js.map