/**
 * @license https://github.com/Intermesh/goui/blob/main/LICENSE MIT License
 * @copyright Copyright 2023 Intermesh BV
 * @author Merijn Schering <mschering@intermesh.nl>
 */
import { Field } from "./Field.js";
import { ContainerField } from "./ContainerField.js";
import { createComponent } from "../Component.js";
import { btn } from "../Button";
/**
 * Field to return an array with objects
 *
 * @see Form
 */
export class ArrayField extends ContainerField {
    /**
     *
     * @param buildField Function that returns a new form field for an array item
     */
    constructor(buildField) {
        super("div");
        this.buildField = buildField;
    }
    set value(v) {
        super.value = v;
        this.items.clear();
        if (v) {
            v.forEach((item) => {
                this.internalAddValue(item);
            });
        }
    }
    get value() {
        const v = [];
        this.items.forEach((item) => {
            if (item instanceof Field) {
                v.push(item.value);
            }
        });
        return v;
    }
    /**
     * Add value to the values array. Also fires change event
     *
     * @param value
     */
    addValue(value = {}) {
        if (!this.oldValue) {
            this.captureValueForChange();
        }
        this.internalAddValue(value);
        this.fireChange();
        return this;
    }
    internalAddValue(value) {
        const field = this.buildField(value);
        field.value = value;
        field.items.add(btn({
            // style: {
            // 	alignSelf: "center"
            // },
            icon: "delete",
            title: "Delete",
            handler: (btn) => {
                btn.parent.remove();
                this.fireChange();
            }
        }));
        this.items.add(field);
    }
    reset() {
        super.reset();
        this.items.clear();
    }
}
/**
 * Shorthand function to create {@see ArrayField}
 *
 * @param config
 * @param items
 */
export const arrayfield = (config, ...items) => createComponent(new ArrayField(config.buildField), config, items);
//# sourceMappingURL=ArrayField.js.map